#include "pika_bsp.h"
#include "pika_config.h"
#include "main.h"
#include <stdio.h>
/* support interrupt */

void __platform_disable_irq_handle()
{
    __disable_irq();
}
void __platform_enable_irq_handle()
{
    __enable_irq();
}


/* support printf */
void HARDWARE_PRINTF_Init(void)
{
    USART_InitType USART_InitStructure;
    GPIO_InitType GPIO_InitStructure;
    /* Peripheral clock enable */
    /* Enable GPIO clock */
    RCC_EnableAPB2PeriphClk(RCC_APB2_PERIPH_GPIOA  | RCC_APB2_PERIPH_AFIO, ENABLE);
    RCC_EnableAPB2PeriphClk(RCC_APB2_PERIPH_USART1, ENABLE);

    /**USART1 GPIO Configuration
    PA9   ------> USART1_TX
    PA10   ------> USART1_RX
    */


    /* Configure USARTy Rx as input floating */
    GPIO_InitStructure.Pin       = GPIO_PIN_10;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_InitPeripheral(GPIOA, &GPIO_InitStructure);

    /* Configure USARTy Tx as alternate function push-pull */
    GPIO_InitStructure.Pin        = GPIO_PIN_9;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_PP;
    GPIO_InitPeripheral(GPIOA, &GPIO_InitStructure);



    /* USART1 interrupt Init */

    NVIC_InitType NVIC_InitStructure;
    /* Configure the NVIC Preemption Priority Bits */
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);

    /* Enable the USARTy Interrupt */
    NVIC_InitStructure.NVIC_IRQChannel            = USART1_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd         = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    NVIC_SetPriority(USART1_IRQn, 0);
    NVIC_EnableIRQ(USART1_IRQn);

    USART_InitStructure.BaudRate            = 115200;
    USART_InitStructure.WordLength          = USART_WL_8B;
    USART_InitStructure.StopBits            = USART_STPB_1;
    USART_InitStructure.Parity              = USART_PE_NO;
    USART_InitStructure.HardwareFlowControl = USART_HFCTRL_NONE;
    USART_InitStructure.Mode                = USART_MODE_RX | USART_MODE_TX;

    /* Configure USARTy and USARTz */
    USART_Init(USART1, &USART_InitStructure);

//    /* Polling USART1 initialisation */
//    while((!(LL_USART_IsActiveFlag_TEACK(USART1))) ||
//            (!(LL_USART_IsActiveFlag_REACK(USART1))))
//    {
//    }

    /* open interrupt */
    /* Enable USARTy Receive and Transmit interrupts */
    USART_ConfigInt(USART1, USART_INT_RXDNE, ENABLE);
//	USART_ConfigInt(USART1, USART_INT_PEF, ENABLE);

    /* Enable the USARTy and USARTz */
    USART_Enable(USART1, ENABLE);

}

int fputc(int ch, FILE* f)
{
    USART_SendData(USART1, (uint8_t)ch);
    while(USART_GetFlagStatus(USART1, USART_FLAG_TXDE) == RESET)
        ;
    return ch;
}

/* support delay_us */

void delay_us(uint32_t udelay)
{
    uint32_t startval, tickn, delays, wait;

    startval = SysTick->VAL;
    tickn = HAL_GetTick();
    delays = udelay * 64;  // delay 1us when delays = 64
    if(delays > startval)
    {
        while(HAL_GetTick() == tickn)
        {
        }
        wait = 64000 + startval - delays;
        while(wait < SysTick->VAL)
        {
        }
    }
    else
    {
        wait = startval - delays;
        while(wait < SysTick->VAL && HAL_GetTick() == tickn)
        {
        }
    }
}


uint32_t globalWriteAddress = 0;
uint32_t GetPage(uint32_t Addr)
{
    return (Addr - FLASH_BASE) / FLASH_PAGE_SIZE ;
}

/* support download python script by uart1 */
uint8_t pika_memory_lock = 0;
uint8_t __is_locked_pikaMemory(void)
{
    return pika_memory_lock;
}

CodeHeap codeHeap;
void MCU_Code_Init(void)
{
    codeHeap.size = 0;
    codeHeap.content = pikaMalloc(codeHeap.size + 1);
    codeHeap.ena = 0;
}

uint8_t MCU_Code_reciveHandler(char* data, uint32_t rxSize)
{
    char buff[RX_BUFF_LENGTH] = {0};
    if(0 == codeHeap.ena)
    {
        char* strLine = strGetLastLine(buff, data);
        if(strIsStartWith(strLine, "import "))
        {
            codeHeap.reciveTime = uwTick;
            codeHeap.ena = 1;
            data = strLine;
            codeHeap.content = malloc(FLASH_SCRIPT_END_ADDR - FLASH_SCRIPT_START_ADDR);
        }
    }
    if(1 == codeHeap.ena)
    {
        if(!pika_memory_lock)
        {
            pika_memory_lock = 1;
        }
        codeHeap.reciveTime = uwTick;
        codeHeap.oldSize = codeHeap.size;
        codeHeap.size += rxSize;
//        codeHeap.content = realloc(codeHeap.content, codeHeap.size + 1);
        memcpy(codeHeap.content + codeHeap.oldSize, data, rxSize);
        codeHeap.content[codeHeap.size] = 0;
        /* reciving code */
        return 1;
    }
    /* not work */
    return 0;
}

void MCU_Code_flashHandler(void)
{
    if(!codeHeap.ena)
    {
        /* recive not activate */
        return;
    }
    if(uwTick - codeHeap.reciveTime < 200)
    {
        /* still reciving */
        return;
    }

    /* transmite is finished */
    MCU_Code_reciveHandler("\n\n", 2);


    uint32_t FirstPage = 0, NbOfPages = 0;
    uint32_t PageError = 0;
    __IO uint32_t data32 = 0, MemoryProgramStatus = 0;
//    static FLASH_EraseInitTypeDef EraseInitStruct = {0};

    printf("==============[Programer]==============\r\n");
    printf("[info]: Recived byte: %d\r\n", codeHeap.size);
    printf("[info]: Programing... \r\n");
    FLASH_Unlock();
    /* Get the 1st page to erase */
    FirstPage = GetPage(FLASH_SCRIPT_START_ADDR);
    printf("startAddr: 0x%08X, page:%d\r\n",FLASH_SCRIPT_START_ADDR,FirstPage);

    /* Get the number of pages to erase from 1st page */
    NbOfPages = GetPage(FLASH_SCRIPT_END_ADDR) - FirstPage + 1;
    printf("endAddr: 0x%08X,number of pages: %d\r\n",FLASH_SCRIPT_END_ADDR,NbOfPages);


    printf("[info]: Erasing flash... \r\n");
 

	for(uint32_t i=0;i<NbOfPages;i++)
	{
		if (FLASH_COMPL != FLASH_EraseOnePage(FLASH_SCRIPT_START_ADDR+i*FLASH_PAGE_SIZE))
		{
			printf("[error]: Erase faild! \r\n");
			while(1)
			{
			}
		}
	}
    printf("[ OK ]: Erase flash ok! \r\n");

    printf("[info]: Writing flash... \r\n");
    uint32_t baseAddress = FLASH_SCRIPT_START_ADDR;
    uint32_t writeAddress = 0;
    uint64_t writeData64 = 0;
	uint32_t writeData32 = 0;
    while(writeAddress < codeHeap.size + 1)
    {

		
		writeData32 = 0; 
        for(int i = 3; i >= 0; i--)
        {
            char ch = codeHeap.content[writeAddress + i];
            writeData32 = writeData32 << 8;
            writeData32 += ch;
			
        }
//        if(HAL_FLASH_Program(FLASH_TYPEPROGRAM_DOUBLEWORD,baseAddress + writeAddress,writeData64) == HAL_OK)
		
 
        if (FLASH_COMPL == FLASH_ProgramWord(baseAddress + writeAddress, writeData32))        	
        {
            writeAddress = writeAddress + 4;
        }
        else
        {
            printf("[error]: Write flash faild. \r\n");
            while(1)
            {
            }
        }
		
		
    }
    FLASH_Lock();
    printf("[ OK ]: Write flash ok! \r\n");

    baseAddress = FLASH_SCRIPT_START_ADDR;
    MemoryProgramStatus = 0x0;

    printf("[info]: Checking flash... \r\n");
    char* codeInFlash = (char*)baseAddress;
    printf("\r\n");
    printf("----[code in flash]-----\r\n");
    for(int i = 0; i < strGetSize(codeHeap.content); i++)
    {
        if('\n' == codeHeap.content[i] && '\r' != codeHeap.content[i - 1])
        {
            fputc('\r', (FILE*)!NULL);
        }
        fputc(codeHeap.content[i], (FILE*)!NULL);
    }
    printf("----[code in flash]-----\r\n");
    printf("\r\n");

    if(!strEqu(codeInFlash, codeHeap.content))
    {
        printf("[error]: Check flash faild.\r\n");
        printf("\r\n");

        printf("\r\n\r\n");
        printf("---------[code in heap]----------\r\n");
        printf("\r\n");
        for(int i = 0; i < strGetSize(codeHeap.content); i++)
        {
            if('\n' == codeHeap.content[i])
            {
                fputc('\r', (FILE*)!NULL);
            }
            fputc(codeHeap.content[i], (FILE*)!NULL);
        }
        printf("\r\n\r\n");
        printf("---------[code in heap]----------\r\n");
        while(1)
        {
        }
    }
    printf("[ OK ]: Checking flash ok! \r\n");
    printf("[ OK ]: Programing ok! \r\n");
    printf("[info]: Restarting... \r\n");
    printf("==============[Programer]==============\r\n");

    printf("\r\n");
    /* System Reset */
    NVIC_SystemReset();
}
